class TerraformBlock:
    def __init__(self):
        """
        Initializes a new instance of the TerraformBlock class.

        Attributes:
            file_path (str): The path of the file where the block is defined
        """
        self.file_path: str = ""
